<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class OISBizcraft extends PaymentModule
{
    protected $_postErrors = array();
    public $merchant_outlet_id;
    public $terminal_id;
    public $cust_code;
    public $secretkey;
    public $portalURL;

    public function __construct()
    {
        $this->name = 'OISBizcraft';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'OIS Bizcraft';
        $this->need_instance = 0;

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('OIS Bizcraft');
        $this->description = $this->l('This is a Prestashop module for secure payments via OIS Bizcraft.');
        $this->confirmUninstall = $this->l('Are you sure about removing these details?');
        $this->limited_currencies = array('EUR', 'SGD');
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false)
        {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        Configuration::updateValue('OISBIZCRAFT_LIVE_MODE', false);
    
        if(!$this->installOrderState())
        {
            return false;
        }

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('paymentReturn');
    }

    public function uninstall()
    {
        if(!Configuration::deleteByName('OISBIZCRAFT_LIVE_MODE')    ||
            !Configuration::deleteByName('OISBIZCRAFT_CUST_CODE')   ||
            !Configuration::deleteByName('OISBIZCRAFT_MERCHANT_ID') ||
            !Configuration::deleteByName('OISBIZCRAFT_TERMINAL_ID') ||
            !Configuration::deleteByName('OISBIZCRAFT_SECRET_KEY')  ||
            !Configuration::deleteByName('OISBIZCRAFT_PORTAL_URL'))
        {
            return false;
        }

        return parent::uninstall();
    }

    public function installOrderState()
    {
        if(!Configuration::get('OISBIZCRAFT_PENDING') || !Validate::isLoadedObject(new OrderState(Configuration::get('OISBIZCRAFT_PENDING'))))
        {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach(Language::getLanguages() as $language)
            {
                $orderState->name[$language['id_lang']] = 'OIS Pending';
            }

            $orderState->module_name    = $this->name;
            $orderState->send_email     = false;
            $orderState->color          = '#4169E1';
            $orderState->hidden         = false;
            $orderState->delivery       = false;
            $orderState->logable        = false;
            $orderState->invoice        = false;

            $orderState->add();
            Configuration::updateValue('OISBIZCRAFT_PENDING', (int) $orderState->id);
        }

        return true;
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitOISBizcraftModule')) == true) 
        {
            $pass_required_validation = $this->validateRequiredConfigFormValues();
            // if validation fails, display error msg
            if ($pass_required_validation == false) {                
                $this->context->controller->errors[] = $this->l('Please fill in all the required data.');                
            } else {                
                $this->postProcess();
                $this->context->controller->confirmations[] = $this->l('Saved with success!');
            }
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitOISBizcraftModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Mode'),
                        'name' => 'OISBIZCRAFT_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live or test mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Live mode')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Test mode')
                            )
                        ),
                    ),
                    array(
                        'col' => 4,
                        'type' => 'text',
                        'label' => $this->l('Secret Key'),
                        'name' => 'OISBIZCRAFT_SECRET_KEY',
                        'desc' => $this->l('You will need the Secret Key which can be obtained from OIS Bizcraft after your ecommerce account has been created.'),
                        'required' => true
                    ),
                    array(
                        'col' => 4,
                        'type' => 'text',
                        'label' => $this->l('Merchant Outlet ID'),
                        'desc' => $this->l('The merchant outlet ID issued by OIS Bizcraft.'),
                        'name' => 'OISBIZCRAFT_MERCHANT_ID',
                        'required' => true
                    ),
                    array(
                        'col' => 4,
                        'type' => 'text',
                        'label' => $this->l('Terminal ID'),
                        'desc' => $this->l('The terminal ID issued by OIS Bizcraft.'),
                        'name' => 'OISBIZCRAFT_TERMINAL_ID',
                        'required' => true
                    ),
                    array(
                        'col' => 4,
                        'type' => 'text',
                        'label' => $this->l('Customer Code'),
                        'desc' => $this->l('The customer code issued by OIS Bizcraft.'),
                        'name' => 'OISBIZCRAFT_CUST_CODE',
                        'required' => true
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'OISBIZCRAFT_LIVE_MODE' => Configuration::get('OISBIZCRAFT_LIVE_MODE', true),
            'OISBIZCRAFT_SECRET_KEY' => Configuration::get('OISBIZCRAFT_SECRET_KEY'),
            'OISBIZCRAFT_MERCHANT_ID' => Configuration::get('OISBIZCRAFT_MERCHANT_ID'),
            'OISBIZCRAFT_TERMINAL_ID' => Configuration::get('OISBIZCRAFT_TERMINAL_ID'),
            'OISBIZCRAFT_CUST_CODE' => Configuration::get('OISBIZCRAFT_CUST_CODE')
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) 
        {
            Configuration::updateValue($key, Tools::getValue($key));
        }

        /**
         * Whenever the form is submitted, we check the value of OISBIZCRAFT_LIVE_MODE
         * and set portal URL accordingly
         */
        if (Configuration::get('OISBIZCRAFT_LIVE_MODE') == true) {
            Configuration::updateValue('OISBIZCRAFT_PORTAL_URL', "https://portalapi.oisbizcraft.com");
        } else {
            Configuration::updateValue('OISBIZCRAFT_PORTAL_URL', "https://devapiportal.oisbizcraft.com");
        }        
    }

    /**
     * Function to validate Prestashop BO form to ensure that required fields are not left empty
     * @return return boolean value depending on whether it passes the validation
     */
    protected function validateRequiredConfigFormValues()
    {
        $oisbizcraft_secret_key = Tools::getValue('OISBIZCRAFT_SECRET_KEY');
        $oisbizcraft_merchant_id = Tools::getValue('OISBIZCRAFT_MERCHANT_ID');
        $oisbizcraft_terminal_id = Tools::getValue('OISBIZCRAFT_TERMINAL_ID');
        $oisbizcraft_cust_code = Tools::getValue('OISBIZCRAFT_CUST_CODE');

        if ($oisbizcraft_secret_key == "" || $oisbizcraft_merchant_id == "" || $oisbizcraft_terminal_id == "" || $oisbizcraft_cust_code == "") {
            return false;
        }
        return true;
    }    

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) 
        {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) 
        {
            return;
        }

        $payment_options = 
        [
            $this->getExternalPaymentOption($params),
        ];

        return $payment_options;
    }

    public function getExternalPaymentOption($params)
    {
        $externalOption = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();

        $externalOption->setCallToActionText($this->l('Credit / Debit Card and QR Code'))
                    ->setAction($this->context->link->getModuleLink($this->name, 'redirect', array(), true))
                    ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/payment-logo.png'))
                    ->setModuleName($this->name);

        return $externalOption;
    }

    public function hookPaymentReturn($params)
    {
        if(!$this->active)
            return;

        $state = $params['order']->getCurrentState();
        $ois_pending_state = Configuration::get('OISBIZCRAFT_PENDING');

        if($state == $ois_pending_state)
        {
            $merchant_outlet_id = Configuration::get('OISBIZCRAFT_MERCHANT_ID');
            $terminal_id = Configuration::get('OISBIZCRAFT_TERMINAL_ID');
            $cust_code = Configuration::get('OISBIZCRAFT_CUST_CODE');
            $portalURL = Configuration::get('OISBIZCRAFT_PORTAL_URL');
            $order_id = $params['order']->id;

            // Hash String
            $tmpString = $cust_code . $merchant_outlet_id . $terminal_id . $order_id;
            $hash_string = strtoupper(hash_hmac('SHA1', $tmpString, Configuration::get('OISBIZCRAFT_SECRET_KEY')));

            // Manual Transaction Status Check
            $postVars = 
            [
                'merchant_outlet_id'    => $merchant_outlet_id,
                'terminal_id'           => $terminal_id,
                'order_id'              => $order_id,
                'hash_string'           => $hash_string,
            ];

            $headers = 
            [
                'Accept: application/json',     
                'Content-Type: application/json',
            ];
            
            $api_url = $portalURL . '/api/payments/' . $cust_code . '/transaction-status';

            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $api_url,         
                CURLOPT_RETURNTRANSFER => true,      
                CURLOPT_ENCODING => '',     
                CURLOPT_MAXREDIRS => 10,      
                CURLOPT_TIMEOUT => 0,     
                CURLOPT_FOLLOWLOCATION => true,   
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,      
                CURLOPT_CUSTOMREQUEST => 'POST',     
                CURLOPT_POSTFIELDS => json_encode($postVars),   
                CURLOPT_HTTPHEADER => $headers,
            ));

            $response = curl_exec($curl);
            curl_close($curl);

            // Check this order payment status
            $paymentStatus = json_decode($response)->data->status;
            $orderHistory = new OrderHistory();
            if($paymentStatus == 'A')
            {
                $order_payment_accept = 2;
                $orderHistory->id_order = (int) $order_id;
                $orderHistory->changeIdOrderState($order_payment_accept, $order_id);
                $orderHistory->add(true);
            }
            else if($paymentStatus == 'F')
            {
                $order_payment_cancel = 8;
                $orderHistory->id_order = (int) $order_id;
                $orderHistory->changeIdOrderState($order_payment_cancel, $order_id);
                $orderHistory->add(true);                
            }
        }
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Generating hashed string of transaction data
     * @param array         the transaction data
     * @return string       returning uppercase of the hashed string
     */
    public function generateHash($data)
    {
        $string = $data['cust_code'] . $data['merchant_outlet_id'] . $data['terminal_id'] . $data['merchant_return_url'] . $data['description'] . $data['currency'] . $data['amount'] . $data['order_id'] . $data['user_fullname'];

        $hash = strtoupper(hash_hmac('SHA1', $string, $data['secret_key']));

        return $hash;
    }

    /**
     * Generating validation keys for the hash key received from OIS
     *
     * @param [array] $data     the data from OIS
     * @return string           returning uppercase of the hashed validate hash string
     */
    public function generateValidateKey($data)
    {
        $string = Configuration::get('OISBIZCRAFT_TERMINAL_ID') . $data['ref_order_id'] . Configuration::get('OISBIZCRAFT_MERCHANT_ID') . Configuration::get('OISBIZCRAFT_CUST_CODE') . $data['status'];
        
        $hash = strtoupper(hash_hmac('SHA1', $string, Configuration::get('OISBIZCRAFT_SECRET_KEY')));

        return $hash;
    }
}
