<?php
class OISBizcraftCallbackModuleFrontController extends ModuleFrontController
{
    /**
     * @see FrontController::postProcess()
     */
    public function postProcess()
    {
        // Create an object of Prestashop FileLogger
        $logger = new FileLogger();
        // File path to write logs
        $filePath = _PS_MODULE_DIR_ . 'OISBizcraft/process.log';
        $logger->setFileName($filePath);

        $response = 
        [
            'success'   => true,
            'message'   => 'Success',
            'status'    => 200,
        ];

        try
        {
            header('Content-Type: application/json;');
            $data = json_decode(file_get_contents('php://input'), true);

            // Get the data from OIS
            // Callback payload in API Document is not accurate: This is enough for prestashop
            $order_id = $data['order_id'];
            $ref_order_id = $data['ref_order_id'];
            $transaction_code = $data['transaction_code'];
            $status = $data['status'];
            $hash = $data['hash_key'];

            // Validation for the required parameters: hash key, order_id and status
            if($order_id == '' || $status == '' || $hash == '')
            {
                $error = 'Either order id, status or hash key is empty';
                $logger->logWarning('[callback.php] ' . $error);
            
                $response['success'] = false;
                $response['message'] = $error;
                $response['status'] = 500;

                http_response_code($response['status']);
                echo json_encode($response);
                exit;
            }
            else
                $logger->logInfo('[callback.php] Data received from OIS.' . json_encode($data));

            // Authentication using HMAC
            $validateKey = OISBizcraft::generateValidateKey($data);
            if($validateKey != $hash)
            {
                $error = 'Validation keys do not match. Generated Hash Key: ' . $validateKey;
                $logger->logWarning('[callback.php] ' . $error);

                $response['success'] = false;
                $response['message'] = $error;
                $response['status'] = 500;

                http_response_code($response['status']);
                echo json_encode($response);    
                exit;
            }
            
            $objOrder = new Order($order_id);

            if(!Validate::isLoadedObject($objOrder))
            {
                $error = 'Unable to find order with this order_id: ' . $order_id;
                $logger->logWarning('[callback.php] ' . $error);

                $response['success'] = false;
                $response['message'] = $error;
                $response['status'] = 500;

                http_response_code($response['status']);
                echo json_encode($response);
                exit;
            }

            $customer = new Customer($objOrder->id_customer);
            $orderHistory = new OrderHistory();
            $obj = new OISBizcraft();

            if(!Validate::isLoadedObject($customer))
            {
                $error = 'Unable to find customer with this id_customer: ' . $objOrder->id_customer;
                $logger->logWarning('[callback.php] ' . $error);

                $response['success'] = false;
                $response['message'] = $error;
                $response['status'] = 500;

                http_response_code($response['status']);
                echo json_encode($response);
                exit;
            }

            // Check payment status
            if(strcasecmp($status, "success") == 0)
            {
                $order_payment_accept = 2;
                $orderHistory->id_order = (int) $objOrder->id;
                $orderHistory->changeIdOrderState($order_payment_accept, $objOrder->id);
                $orderHistory->add(true);

                $order_payments = OrderPayment::getByOrderReference($objOrder->reference);
                if (isset($order_payments[0])) {
                    $order_payments[0]->transaction_id = $transaction_code;
                    $order_payments[0]->save();
                }

                $this->removeCart();
            }
            else if(strcasecmp($status, "cancelled") == 0)
            {
                $order_payment_cancel = 6;
                $orderHistory->id_order = (int) $objOrder->id;
                $orderHistory->changeIdOrderState($order_payment_cancel, $objOrder->id);
                $orderHistory->add(true);
            }
            else if(strcasecmp($status, "failed") == 0)
            {
                $order_payment_cancel = 8;
                $orderHistory->id_order = (int) $objOrder->id;
                $orderHistory->changeIdOrderState($order_payment_cancel, $objOrder->id);
                $orderHistory->add(true);
            }
        }
        catch(\Throwable $th)
        {
            $logger->logWarning('[callback.php] Errors: ' . $th);

            $response['success'] = false;
            $response['message'] = 'Error';
            $response['status'] = 500;
        }

        http_response_code($response['status']);
        echo json_encode($response);
        exit;
    }
                               
    function removeCart() 
    {
        $this->context->cart->delete();
    }
}