<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/
class OISBizcraftRedirectModuleFrontController extends ModuleFrontController
{
    /**
     * Do whatever you have to before redirecting the customer on the website of your payment processor.
     */
    public function postProcess()
    {
        // Store the login information into cookie for further use
        $this->context->cookie->__set('isLogged', $this->context->customer->isLogged());

        $cart = $this->context->cart;
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) 
        {
            Tools::redirect('index.php?controller=order&step=1');
        }
        
        // Check that this payment option is still available in case the customer changed the address just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) 
        {
            if ($module['name'] == 'OISBizcraft') 
            {
                $authorized = true;
                break;
            }
        }
        if (!$authorized) 
        {
            die($this->module->l('This payment method is not available.', 'validation'));
        }
        
        // Getting customer full name
        $customer = new Customer($cart->id_customer);
        $customer_fullname = $customer->firstname . ' ' . $customer->lastname;
        if (!Validate::isLoadedObject($customer))
        {
            Tools::redirect('index.php?controller=order&step=1');
        }
        
        // Getting customer checkout email
        $checkout_email = $this->context->cookie->email;
        
        // Get currency iso code (SGD e.g.)
        $currency = $this->context->currency;
        $currency_iso_code = $currency->iso_code;
        
        // Get cart total amount
        // Need to multiply by 100 and make it INT type
        $tmpTotal = (float) $cart->getOrderTotal(true, Cart::BOTH);
        $total = (int)round(($tmpTotal * 100), 0);

        // Put a description from the cart's products for this payment
        $description = 'Order made with OIS Bizcraft at Prestashop.';
        
        // Email variables
        $mailVars = array(
            '{merchant_outlet_id}' => Configuration::get('OISBIZCRAFT_MERCHANT_ID'),
            '{terminal_id}' => Configuration::get('OISBIZCRAFT_TERMINAL_ID'),
        );
    
        // Store the cart id into cookie for further use
        if(isset($cart->id))
        {
            $this->context->cookie->__set('cart_id', $cart->id);
        }

        $cart_id = $this->context->cookie->__get('cart_id');
        $order_id = Order::getOrderByCartId($cart_id);
    
        // If there is no order id, we will create one with the custom order status of "OISBIZCRAFT_PENDING".
        if(!$order_id)
        {
            $order_ois_pending = Configuration::get('OISBIZCRAFT_PENDING');
            $result = $this->module->validateOrder((int) $cart->id, $order_ois_pending, $total, $this->module->displayName, NULL, $mailVars, (int) $currency->id, false, $customer->secure_key);
        }
        
        $order_id = Order::getOrderByCartId($cart_id);

        // Restore cart with the previous cart_id
        $this->restoreCart($cart_id);

        // Generate a return URL using Prestashop getModuleLink
        // OIS will redirect customer to this URL which would be:
        // [domain]/[OISBizcraft]/[response]
        // If there are no URL parameters then need the '?', else must remove
        $returnURL = $this->context->link->getModuleLink('OISBizcraft', 'response') . '?';

        // Callback URL
        // This is the URL where OIS will send us the payment status
        $callbackURL = $this->context->link->getModuleLink('OISBizcraft', 'callback') . '?';

        // Send an API call to OIS Bizcraft to get the URL        
        // data for the hash function
        $toHashData = array(
            'order_id'              => strval($order_id),
            'merchant_outlet_id'    => strval(Configuration::get('OISBIZCRAFT_MERCHANT_ID')),
            'terminal_id'           => strval(Configuration::get('OISBIZCRAFT_TERMINAL_ID')),
            'cust_code'             => strval(Configuration::get('OISBIZCRAFT_CUST_CODE')),
            'merchant_return_url'   => $returnURL,
            'backend_callback_url'  => $callbackURL,
            'amount'                => (int)$total,
            'currency'              => strval($currency_iso_code),
            'user_fullname'         => strval($customer_fullname),
            'user_email'            => strval($checkout_email),
            'description'           => $description,
            'secret_key'            => strval(Configuration::get('OISBIZCRAFT_SECRET_KEY'))
        );
        
        $hash = OISBizcraft::generateHash($toHashData);

        // cURL POST vars
        $postVars = array(
            'order_id'              => $toHashData['order_id'],
            'merchant_outlet_id'    => $toHashData['merchant_outlet_id'],
            'terminal_id'           => $toHashData['terminal_id'],
            'cust_code'             => $toHashData['cust_code'],
            'merchant_return_url'   => $toHashData['merchant_return_url'],
            'backend_callback_url'  => $toHashData['backend_callback_url'],
            'amount'                => $toHashData['amount'],
            'currency'              => $toHashData['currency'],
            'user_fullname'         => $toHashData['user_fullname'],
            'user_email'            => $toHashData['user_email'],
            'description'           => $toHashData['description'],
            'hash'                  => $hash,
            'origin_call'           => 'prestashop'
        );

        $headers = array(
            'Accept: application/json',     
            'Content-Type: application/json',
        );

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => Configuration::get('OISBIZCRAFT_PORTAL_URL') . '/api/payments',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => json_encode($postVars),
            CURLOPT_HTTPHEADER => $headers,
        ));
            
        $response = curl_exec($curl);
        curl_close($curl);       

        $finalURL = '';

        $decodedURL = json_decode($response)->data->url;
        $explodedArr = explode('&', $decodedURL);
        foreach($explodedArr as $key => $value)
        {
            if($key == 0)
            {
                $finalURL .= $value;
            }
            else
            {
                $finalURL .= '&' . $value;
            }
        }

        // Unset value from cookies
        $this->context->cookie->__unset('cart_id');
        
        Tools::redirect($finalURL);

        /*
         * An error occured.
         */
        if (Tools::getValue('action') == 'error') 
        {
            return $this->displayError('An error occurred while trying to redirect the customer');
        } 
        else 
        {
            $this->context->smarty->assign(array(
                'cart_id' => Context::getContext()->cart->id,
                'secure_key' => Context::getContext()->customer->secure_key,
            ));

            return $this->setTemplate('redirect.tpl');
        }
    }

    protected function displayError($message, $description = false)
    {
        /*
         * Create the breadcrumb for your ModuleFrontController.
         */
        $this->context->smarty->assign('path', '
			<a href="' . $this->context->link->getPageLink('order', null, null, 'step=3') . '">' . $this->module->l('Payment') . '</a>
			<span class="navigation-pipe">&gt;</span>' . $this->module->l('Error'));

        /*
         * Set error message and description for the template.
         */
        array_push($this->errors, $this->module->l($message), $description);

        return $this->setTemplate('error.tpl');
    }

    // This is to restore cart with the same items as the most recently created order
    // When an order is created, the cart is cleared. If a customer cancel, go back to previous page or payment failed, they will not be able to see their "original" cart.
    private function restoreCart($_cartId)
    {
        $order = new Order(Order::getOrderByCartId($_cartId));

        if($order)
        {
            $oldCart = new Cart($order->id_cart);
            $duplicatedCart = $oldCart->duplicate();

            if(!$duplicatedCart || !Validate::isLoadedObject($duplicatedCart['cart']))
            {
                $this->errors[] = Tools::displayError('Sorry, we are unable to restore your cart.');
            }
            else if(!$duplicatedCart['success'])
            {
                $this->errors[] = Tools::displayError('Some items are no longer available, and we are unable to restore your cart.');
            }
            else
            {
                $this->context->cookie->id_cart = $duplicatedCart['cart']->id;
                $this->context->cart = $duplicatedCart['cart'];
                CartRule::autoAddToCart($this->context);
                $this->context->cookie->write();
            }
        }
    }
}
